@@ -8,9 +8,11 @@ from typing import Any, Dict, List, Optional, Union
 import antlr4.tree.Tree
 import sqlalchemy
 from antlr4 import InputStream, Token
+from sqlalchemy import text
 from sqlalchemy.engine import Connection
+from sqlalchemy.sql.expression import TextClause
 
-from pytsql.grammar import USE_CPP_IMPLEMENTATION, SA_ErrorListener, parse, tsqlParser
+from pytsql.grammar import USE_CPP_IMPLEMENTATION, SA_ErrorListener, TSqlParser, parse
 
 _REPLACE_START = "<replace>"
 _REPLACE_END = "</replace>"
@@ -18,6 +20,8 @@ _EOF = "<EOF>"
 _GO = "GO"
 _PRINTS_TABLE = "#pytsql_prints"
 
+logger = logging.getLogger("pytsql")
+
 
 def _code(path: Union[str, Path], encoding: str) -> str:
     with Path(path).open(encoding=encoding) as fh:
@@ -71,6 +75,14 @@ def _parametrize(
     return "\n".join(non_empty_stripped_lines)
 
 
+def _text(raw_sql: str) -> TextClause:
+    """
+    Calls sa.text() with escaped colons as in this package every colon is required verbatim.
+    """
+    raw_sql_escaped = raw_sql.replace(":", "\\:")
+    return text(raw_sql_escaped)
+
+
 class _TSQLVisitor(antlr4.ParseTreeVisitor):
     def __init__(self):
         # This attribute holds declarations of sql variables, also
@@ -83,22 +95,25 @@ class _TSQLVisitor(antlr4.ParseTreeVisitor):
         # batches.
         self.dynamics: List[str] = []
 
-    def visit(self, tree: tsqlParser.Sql_clauseContext) -> str:
+    def visit(
+        self, tree: TSqlParser.Sql_clausesContext, prepend_dynamics: bool = True
+    ) -> str:
         dynamics = self.dynamics[:]
 
         chunks = tree.accept(self)
 
-        # CREATE SCHEMA must be the only statement in a batch
-        if (
-            tree.ddl_clause() is not None
-            and tree.ddl_clause().create_schema() is not None
-        ):
+        # CREATE SCHEMA/VIEW must be the only statement in a batch
+        is_create_schema_or_view = hasattr(tree, "ddl_clause") and (
+            hasattr(tree.ddl_clause(), "create_schema")
+            or hasattr(tree.ddl_clause(), "create_view")
+        )
+        if not prepend_dynamics or is_create_schema_or_view:
             return " ".join(chunks)
 
         return " ".join(dynamics + chunks)
 
     def visitChildren(self, node: antlr4.ParserRuleContext) -> List[str]:
-        if isinstance(node, tsqlParser.Print_statementContext):
+        if isinstance(node, TSqlParser.Print_statementContext):
             # Print statements are replaced by inserts into a temporary table so that they can be evaluated
             # at the right time and fetched afterwards.
             result = (
@@ -109,7 +124,7 @@ class _TSQLVisitor(antlr4.ParseTreeVisitor):
         else:
             result = super().visitChildren(node)
 
-        if isinstance(node, tsqlParser.Declare_statementContext):
+        if isinstance(node, TSqlParser.Declare_statementContext):
             self.dynamics.extend(result)
 
         return result
@@ -139,11 +154,11 @@ class _RaisingErrorListener(SA_ErrorListener):
         error_message = f"Line {line}:{column} {msg}."
         if offendingSymbol is not None:
             error_message += f" Problem near token '{offendingSymbol.text}'."
-        logging.error(error_message)
+        logger.error(error_message)
         raise ValueError(f"Error parsing SQL script: {error_message}")
 
 
-def _split(code: str) -> List[str]:
+def _split(code: str, isolate_top_level_statements: bool = True) -> List[str]:
     if not USE_CPP_IMPLEMENTATION:
         warnings.warn(
             "Can not find C++ version of the parser, Python version will be used instead."
@@ -151,7 +166,7 @@ def _split(code: str) -> List[str]:
             " and is preventing the use of the faster C++ parser."
         )
 
-    logging.info("Started SQL script parsing")
+    logger.debug("Started SQL script parsing")
 
     # The default error listener only prints to the console without raising exceptions.
     error_listener = _RaisingErrorListener()
@@ -160,32 +175,48 @@ def _split(code: str) -> List[str]:
     tree = parse(InputStream(data=code), "tsql_file", error_listener)
     visitor = _TSQLVisitor()
 
-    # Our current definition of a 'batch' is a single top-level SQL clause.
+    # Our current definition of a 'batch' in isolation mode is a single top-level SQL clause.
     # Note that this differs from the grammar definition of a batch, which is
-    # a group of clauses between GO statements
+    # a group of clauses between GO statements. The latter matches the definition of batches
+    # in non-isolation mode.
     batches = []
     for batch in tree.batch():
-        for sql_clause in batch.sql_clauses().sql_clause():
-            batch_query = visitor.visit(sql_clause)
-            if batch_query != "":
-                batches.append(batch_query)
+        if batch.batch_level_statement() is not None:
+            batch_level_statement = visitor.visit(
+                batch.batch_level_statement(), prepend_dynamics=False
+            )
+            batches.append(batch_level_statement)
+        else:
+            clauses = []
+            first_clause_in_batch = True
+            for sql_clause in batch.sql_clauses():
+                prepend_dynamics = first_clause_in_batch or isolate_top_level_statements
+                clause = visitor.visit(sql_clause, prepend_dynamics=prepend_dynamics)
+                if clause != "":
+                    clauses.append(clause)
+                    first_clause_in_batch = False
+            if isolate_top_level_statements:
+                batches.extend(clauses)
+            else:
+                batches.append("\n".join(clauses))
 
-    logging.info("SQL script parsed successfully.")
+    logger.debug("SQL script parsed successfully.")
 
     return batches
 
 
 def _fetch_and_clear_prints(conn: Connection):
-    prints = conn.execute(f"SELECT * FROM {_PRINTS_TABLE};")
+    prints = conn.execute(_text(f"SELECT * FROM {_PRINTS_TABLE};"))
     for row in prints.all():
-        logging.info(f"Captured PRINT statement: {row[0]}")
-    conn.execute(f"DELETE FROM {_PRINTS_TABLE};")
+        logger.info(f"SQL PRINT: {row[0]}")
+    conn.execute(_text(f"DELETE FROM {_PRINTS_TABLE};"))
 
 
 def executes(
     code: str,
     engine: sqlalchemy.engine.Engine,
     parameters: Optional[Dict[str, Any]] = None,
+    isolate_top_level_statements=True,
 ) -> None:
     """Execute a given sql string through a sqlalchemy.engine.Engine connection.
 
@@ -197,6 +228,7 @@ def executes(
     code T-SQL string to be executed
     engine (sqlalchemy.engine.Engine): established mssql connection
     parameters An optional dictionary of parameters to substituted in the sql script
+    isolate_top_level_statements: whether to execute statements one by one or in whole batches
 
     Returns
     -------
@@ -207,10 +239,11 @@ def executes(
     with engine.connect().execution_options(isolation_level="AUTOCOMMIT") as conn:
         # Since the prints table is a temporary one, it will be local to the connection and it will be dropped once the
         # connection is closed. Caveat: sqlalchemy engines can pool connections, so we still have to drop it preemtively.
-        conn.execute(f"DROP TABLE IF EXISTS {_PRINTS_TABLE}")
-        conn.execute(f"CREATE TABLE {_PRINTS_TABLE} (p NVARCHAR(4000))")
-        for batch in _split(parametrized_code):
-            conn.execute(batch)
+        conn.execute(_text(f"DROP TABLE IF EXISTS {_PRINTS_TABLE}"))
+        conn.execute(_text(f"CREATE TABLE {_PRINTS_TABLE} (p NVARCHAR(4000))"))
+        for batch in _split(parametrized_code, isolate_top_level_statements):
+            sql_batch = _text(batch)
+            conn.execute(sql_batch)
             _fetch_and_clear_prints(conn)
 
 
@@ -218,6 +251,7 @@ def execute(
     path: Union[str, Path],
     engine: sqlalchemy.engine.Engine,
     parameters: Optional[Dict[str, Any]] = None,
+    isolate_top_level_statements=True,
     encoding: str = "utf-8",
 ) -> None:
     """Execute a given sql script through a sqlalchemy.engine.Engine connection.
@@ -227,10 +261,11 @@ def execute(
     path (Path or str): Path to the sql file to be executed
     engine (sqlalchemy.engine.Engine): established mssql connection
     encoding: file encoding of the sql script (default: utf-8)
+    isolate_top_level_statements: whether to execute statements one by one or in whole batches
 
     Returns
     -------
     None
 
     """
-    executes(_code(path, encoding), engine, parameters)
+    executes(_code(path, encoding), engine, parameters, isolate_top_level_statements)
